@@ -22,6 +22,7 @@ import glob
 import os
 import logging
 import socket
+import urllib
 
 from gi.repository import Gio
 from gi.repository import GObject
@@ -34,7 +35,9 @@ from .baseclass import vmmGObjectUI
 HV_XEN,
 HV_LXC,
 HV_QEMU_SESSION,
-HV_BHYVE) = range(5)
+HV_BHYVE,
+HV_VZ,
+HV_CUSTOM) = range(7)
 
 (CONN_SSH,
 CONN_TCP,
@@ -45,7 +48,7 @@ def current_user():
     try:
         import getpass
         return getpass.getuser()
-    except:
+    except Exception:
         return ""
 
 
@@ -98,7 +101,7 @@ class vmmConnect(vmmGObjectUI):
             # Call any API, so we detect if avahi is even available or not
             self.avahiserver.GetAPIVersion()
             logging.debug("Connected to avahi")
-        except Exception, e:
+        except Exception as e:
             self.dbus = None
             self.avahiserver = None
             logging.debug("Couldn't contact avahi: %s", str(e))
@@ -161,22 +164,29 @@ class vmmConnect(vmmGObjectUI):
 
         def _add_hv_row(rowid, config_name, label):
             if (not self.config.default_hvs or
+                not config_name or
                 config_name in self.config.default_hvs):
                 model.append([rowid, label])
 
         _add_hv_row(HV_QEMU, "qemu", "QEMU/KVM")
-        _add_hv_row(HV_QEMU_SESSION, "qemu", "QEMU/KVM user session")
+        _add_hv_row(HV_QEMU_SESSION, "qemu", "QEMU/KVM " + _("user session"))
         _add_hv_row(HV_XEN, "xen", "Xen")
-        _add_hv_row(HV_LXC, "lxc", "LXC (Linux Containers)")
+        _add_hv_row(HV_LXC, "lxc", "LXC (" + _("Linux Containers") + ")")
         _add_hv_row(HV_BHYVE, "bhyve", "Bhyve")
+        _add_hv_row(HV_VZ, "vz", "Virtuozzo")
+        _add_hv_row(-1, None, "")
+        _add_hv_row(HV_CUSTOM, None, "Custom URI...")
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 1)
+        def sepfunc(model, it):
+            return model[it][0] == -1
+        combo.set_row_separator_func(sepfunc)
 
         combo = self.widget("transport")
         model = Gtk.ListStore(str)
         model.append(["SSH"])
         model.append(["TCP (SASL, Kerberos)"])
-        model.append(["SSL/TLS with certificates"])
+        model.append(["SSL/TLS " + _("with certificates")])
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 0)
 
@@ -196,6 +206,7 @@ class vmmConnect(vmmGObjectUI):
         self.widget("hostname").get_child().set_text("")
         self.widget("connect-remote").set_active(False)
         self.widget("username-entry").set_text("")
+        self.widget("uri-entry").set_text("")
         self.connect_remote_toggled(self.widget("connect-remote"))
         self.populate_uri()
 
@@ -230,7 +241,7 @@ class vmmConnect(vmmGObjectUI):
 
             sig = resint.connect("g-signal", cb)
             self.browser_sigs.append((resint, sig))
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
 
     def remove_service(self, interface, protocol, name, typ, domain, flags):
@@ -246,7 +257,7 @@ class vmmConnect(vmmGObjectUI):
             for row in model:
                 if row[0] == name:
                     model.remove(row.iter)
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
 
     def add_conn_to_list(self, interface, protocol, name, typ, domain,
@@ -269,7 +280,7 @@ class vmmConnect(vmmGObjectUI):
 
             host = self.sanitize_hostname(str(host))
             model.append([str(address), str(host), str(name)])
-        except Exception, e:
+        except Exception as e:
             logging.exception(e)
 
     def start_browse(self):
@@ -329,19 +340,26 @@ class vmmConnect(vmmGObjectUI):
     def hypervisor_changed(self, src):
         ignore = src
         hv = uiutil.get_list_selection(self.widget("hypervisor"))
-        is_session = (hv == HV_QEMU_SESSION)
+        is_session = hv == HV_QEMU_SESSION
+        is_custom = hv == HV_CUSTOM
+        show_remote = not is_session and not is_custom
         uiutil.set_grid_row_visible(
             self.widget("session-warning-box"), is_session)
         uiutil.set_grid_row_visible(
-            self.widget("connect-remote"), not is_session)
+            self.widget("connect-remote"), show_remote)
         uiutil.set_grid_row_visible(
-            self.widget("username-entry"), not is_session)
+            self.widget("username-entry"), show_remote)
         uiutil.set_grid_row_visible(
-            self.widget("hostname"), not is_session)
+            self.widget("hostname"), show_remote)
         uiutil.set_grid_row_visible(
-            self.widget("transport"), not is_session)
-        if is_session:
+            self.widget("transport"), show_remote)
+        if not show_remote:
             self.widget("connect-remote").set_active(False)
+
+        uiutil.set_grid_row_visible(self.widget("uri-label"), not is_custom)
+        uiutil.set_grid_row_visible(self.widget("uri-entry"), is_custom)
+        if is_custom:
+            self.widget("uri-entry").grab_focus()
         self.populate_uri()
 
     def username_changed(self, src_ignore):
@@ -363,7 +381,7 @@ class vmmConnect(vmmGObjectUI):
 
     def populate_uri(self):
         uri = self.generate_uri()
-        self.widget("uri-entry").set_text(uri)
+        self.widget("uri-label").set_text(uri)
 
     def populate_default_user(self):
         conn = self.widget("transport").get_active()
@@ -384,12 +402,14 @@ class vmmConnect(vmmGObjectUI):
             hvstr = "qemu"
         elif hv == HV_BHYVE:
             hvstr = "bhyve"
+        elif hv == HV_VZ:
+            hvstr = "vz"
         else:
             hvstr = "lxc"
 
         addrstr = ""
         if user:
-            addrstr += user + "@"
+            addrstr += urllib.quote(user) + "@"
 
         if host.count(":") > 1:
             host = "[%s]" % host
@@ -408,7 +428,7 @@ class vmmConnect(vmmGObjectUI):
             hoststr += addrstr + "/"
 
         uri = hvstr + hoststr
-        if hv in (HV_QEMU, HV_BHYVE):
+        if hv in (HV_QEMU, HV_BHYVE, HV_VZ):
             uri += "system"
         elif hv == HV_QEMU_SESSION:
             uri += "session"
@@ -432,7 +452,10 @@ class vmmConnect(vmmGObjectUI):
         auto = False
         if self.widget("autoconnect").get_sensitive():
             auto = self.widget("autoconnect").get_active()
-        uri = self.generate_uri()
+        if self.widget("uri-label").is_visible():
+            uri = self.generate_uri()
+        else:
+            uri = self.widget("uri-entry").get_text()
 
         logging.debug("Generate URI=%s, auto=%s", uri, auto)
         self.close()
@@ -467,7 +490,7 @@ class vmmConnect(vmmGObjectUI):
             elif self.can_resolve_local is None:
                 try:
                     socket.getaddrinfo(host, None)
-                except:
+                except Exception:
                     logging.debug("Couldn't resolve host '%s'. Stripping "
                                   "'.local' and retrying.", host)
                     self.can_resolve_local = False
@@ -481,7 +504,7 @@ class vmmConnect(vmmGObjectUI):
             elif self.can_resolve_hostname is None:
                 try:
                     socket.getaddrinfo(host, None)
-                except:
+                except Exception:
                     logging.debug("Couldn't resolve host '%s'. Disabling "
                                   "host name resolution, only using IP addr",
                                   host)
