@@ -18,18 +18,16 @@
 # MA 02110-1301 USA.
 #
 
-# pylint: disable=E0611
+import logging
+
 from gi.repository import Gtk
 from gi.repository import Gdk
-# pylint: enable=E0611
-
-import logging
 
 from virtinst import Interface, InterfaceProtocol
 
-from virtManager import uiutil
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.asyncjob import vmmAsyncJob
+from . import uiutil
+from .baseclass import vmmGObjectUI
+from .asyncjob import vmmAsyncJob
 
 PAGE_TYPE = 0
 PAGE_DETAILS = 1
@@ -79,12 +77,12 @@ class vmmCreateInterface(vmmGObjectUI):
         self.ip_manually_changed = False
 
         self.builder.connect_signals({
-            "on_vmm_create_interface_delete_event" : self.close,
+            "on_vmm_create_interface_delete_event": self.close,
 
             "on_cancel_clicked": self.close,
-            "on_back_clicked" : self.back,
-            "on_forward_clicked" : self.forward,
-            "on_finish_clicked" : self.finish,
+            "on_back_clicked": self.back,
+            "on_forward_clicked": self.forward,
+            "on_finish_clicked": self.finish,
             "on_pages_switch_page": self.page_changed,
 
             "on_bridge_config_button_clicked": self.show_bridge_config,
@@ -94,7 +92,7 @@ class vmmCreateInterface(vmmGObjectUI):
 
             # Bridge config dialog
             "on_bridge_config_delete_event": self.bridge_config_finish,
-            "on_bridge_ok_clicked" : self.bridge_config_finish,
+            "on_bridge_ok_clicked": self.bridge_config_finish,
 
             # IP config dialog
             "on_ip_config_delete_event": self.ip_config_finish,
@@ -111,7 +109,7 @@ class vmmCreateInterface(vmmGObjectUI):
 
             # Bond config dialog
             "on_bond_config_delete_event": self.bond_config_finish,
-            "on_bond_ok_clicked" : self.bond_config_finish,
+            "on_bond_ok_clicked": self.bond_config_finish,
 
             "on_bond_monitor_mode_changed": self.bond_monitor_mode_changed,
         })
@@ -126,20 +124,24 @@ class vmmCreateInterface(vmmGObjectUI):
         self.topwin.present()
 
     def show_bond_config(self, src):
+        ignore = src
         logging.debug("Showing new interface bond config")
         self.bond_config.show_all()
 
     def show_bridge_config(self, src):
+        ignore = src
         logging.debug("Showing new interface bridge config")
         self.bridge_config.show_all()
 
     def show_ip_config(self, src):
+        ignore = src
         logging.debug("Showing new interface ip config")
         self.ip_manually_changed = True
         self.ip_config.show_all()
 
     def close(self, ignore1=None, ignore2=None):
-        logging.debug("Closing new interface wizard")
+        if self.topwin.is_visible():
+            logging.debug("Closing new interface wizard")
         self.ip_config.hide()
         self.bridge_config.hide()
         self.bond_config.hide()
@@ -168,8 +170,7 @@ class vmmCreateInterface(vmmGObjectUI):
     @staticmethod
     def iface_in_use_by(conn, name):
         use_str = ""
-        for i in conn.list_interface_names():
-            iface = conn.get_interface(i)
+        for iface in conn.list_interfaces():
             if name in iface.get_slave_names():
                 if use_str:
                     use_str += ", "
@@ -181,7 +182,7 @@ class vmmCreateInterface(vmmGObjectUI):
     def build_interface_startmode_combo(combo):
         model = Gtk.ListStore(str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 0)
+        uiutil.init_combo_text_column(combo, 0)
 
         model.append(["none"])
         model.append(["onboot"])
@@ -198,7 +199,7 @@ class vmmCreateInterface(vmmGObjectUI):
         type_list = self.widget("interface-type")
         type_model = Gtk.ListStore(str, str)
         type_list.set_model(type_model)
-        uiutil.set_combo_text_column(type_list, 1)
+        uiutil.init_combo_text_column(type_list, 1)
         type_model.append([Interface.INTERFACE_TYPE_BRIDGE,
                            _("Bridge")])
         type_model.append([Interface.INTERFACE_TYPE_BOND,
@@ -214,7 +215,7 @@ class vmmCreateInterface(vmmGObjectUI):
 
         # Parent/slave Interface list
         slave_list = self.widget("interface-list")
-        # [ vmmInterface, selected, selectabel, name, type, is defined,
+        # [ vmmInterface, selected, selectable, name, type, is defined,
         #   is active, in use by str, mac]
         slave_model = Gtk.ListStore(object, bool, bool, str, str, bool, bool,
                                     str, str)
@@ -258,7 +259,7 @@ class vmmCreateInterface(vmmGObjectUI):
         mode_list = self.widget("bond-mode")
         mode_model = Gtk.ListStore(str, str)
         mode_list.set_model(mode_model)
-        uiutil.set_combo_text_column(mode_list, 0)
+        uiutil.init_combo_text_column(mode_list, 0)
         mode_model.append([_("System default"), None])
         for m in Interface.INTERFACE_BOND_MODES:
             mode_model.append([m, m])
@@ -266,7 +267,7 @@ class vmmCreateInterface(vmmGObjectUI):
         mon_list = self.widget("bond-monitor-mode")
         mon_model = Gtk.ListStore(str, str)
         mon_list.set_model(mon_model)
-        uiutil.set_combo_text_column(mon_list, 0)
+        uiutil.init_combo_text_column(mon_list, 0)
         mon_model.append([_("System default"), None])
         for m in Interface.INTERFACE_BOND_MONITOR_MODES:
             mon_model.append([m, m])
@@ -274,14 +275,14 @@ class vmmCreateInterface(vmmGObjectUI):
         validate_list = self.widget("arp-validate")
         validate_model = Gtk.ListStore(str)
         validate_list.set_model(validate_model)
-        uiutil.set_combo_text_column(validate_list, 0)
+        uiutil.init_combo_text_column(validate_list, 0)
         for m in Interface.INTERFACE_BOND_MONITOR_MODE_ARP_VALIDATE_MODES:
             validate_model.append([m])
 
         carrier_list = self.widget("mii-carrier")
         carrier_model = Gtk.ListStore(str)
         carrier_list.set_model(carrier_model)
-        uiutil.set_combo_text_column(carrier_list, 0)
+        uiutil.init_combo_text_column(carrier_list, 0)
         for m in Interface.INTERFACE_BOND_MONITOR_MODE_MII_CARRIER_TYPES:
             carrier_model.append([m])
 
@@ -289,24 +290,23 @@ class vmmCreateInterface(vmmGObjectUI):
         copy_iface = self.widget("ip-copy-interface-combo")
         copy_model = Gtk.ListStore(str, object, bool)
         copy_iface.set_model(copy_model)
-        uiutil.set_combo_text_column(copy_iface, 0)
-        copy_iface.add_attribute(txt, "sensitive", 2)
+        uiutil.init_combo_text_column(copy_iface, 0)
 
         ip_mode = self.widget("ipv4-mode")
         ip_model = Gtk.ListStore(str)
         ip_mode.set_model(ip_model)
-        uiutil.set_combo_text_column(ip_mode, 0)
+        uiutil.init_combo_text_column(ip_mode, 0)
         ip_model.insert(IP_DHCP, ["DHCP"])
-        ip_model.insert(IP_STATIC, ["Static"])
-        ip_model.insert(IP_NONE, ["No configuration"])
+        ip_model.insert(IP_STATIC, [_("Static")])
+        ip_model.insert(IP_NONE, [_("No configuration")])
 
         ip_mode = self.widget("ipv6-mode")
         ip_model = Gtk.ListStore(str)
         ip_mode.set_model(ip_model)
-        uiutil.set_combo_text_column(ip_mode, 0)
+        uiutil.init_combo_text_column(ip_mode, 0)
         ip_model.insert(IP_DHCP, ["DHCP"])
-        ip_model.insert(IP_STATIC, ["Static"])
-        ip_model.insert(IP_NONE, ["No configuration"])
+        ip_model.insert(IP_STATIC, [_("Static")])
+        ip_model.insert(IP_NONE, [_("No configuration")])
 
         v6_addr = self.widget("ipv6-address-list")
         addr_model = Gtk.ListStore(str)
@@ -320,7 +320,6 @@ class vmmCreateInterface(vmmGObjectUI):
         txt_col.add_attribute(txt, "text", 0)
 
     def reset_state(self):
-
         self.widget("pages").set_current_page(PAGE_TYPE)
         self.page_changed(None, None, PAGE_TYPE)
 
@@ -383,9 +382,9 @@ class vmmCreateInterface(vmmGObjectUI):
 
         # Make sure interface type specific fields are shown
         type_dict = {
-            Interface.INTERFACE_TYPE_BRIDGE : "bridge",
-            Interface.INTERFACE_TYPE_BOND : "bond",
-            Interface.INTERFACE_TYPE_VLAN : "vlan",
+            Interface.INTERFACE_TYPE_BRIDGE: "bridge",
+            Interface.INTERFACE_TYPE_BOND: "bond",
+            Interface.INTERFACE_TYPE_VLAN: "vlan",
         }
 
         for key, value in type_dict.items():
@@ -436,9 +435,9 @@ class vmmCreateInterface(vmmGObjectUI):
             # We only want configured (aka interface API) interfaces with
             # actually present <protocol> info
             if not is_defined or not iface_obj:
-                label += " (Not configured)"
+                label += " (" + _("Not configured") + ")"
             elif not iface_obj.get_protocol_xml():
-                label += " (No IP configuration)"
+                label += " (" + _("No IP configuration") + ")"
             else:
                 sensitive = True
 
@@ -453,7 +452,7 @@ class vmmCreateInterface(vmmGObjectUI):
             copy_model.append(row)
 
         if len(copy_model) == 0:
-            copy_model.append(["No child interfaces selected.", None, False])
+            copy_model.append([_("No child interfaces selected."), None, False])
 
         if not enable_copy:
             copy_model.clear()
@@ -504,15 +503,15 @@ class vmmCreateInterface(vmmGObjectUI):
         self.widget("interface-list-text").set_text(msg)
 
         nodedevs = {}
-        for phys in self.conn.get_nodedevs("net"):
-            nodedevs[phys.interface] = [None,
-                                        False, False, phys.interface,
-                                        "ethernet", False, True, None,
-                                        phys.address]
+        for phys in self.conn.filter_nodedevs("net"):
+            nodedevs[phys.xmlobj.interface] = [None,
+                False, False, phys.xmlobj.interface,
+                "ethernet", False, True, None,
+                phys.xmlobj.address]
 
         row_dict = {}
-        for name in self.conn.list_interface_names():
-            iface = self.conn.get_interface(name)
+        for iface in self.conn.list_interfaces():
+            name = iface.get_name()
             key = iface.get_xmlobj()
             iface_type = iface.get_type()
             active = iface.is_active()
@@ -550,7 +549,7 @@ class vmmCreateInterface(vmmGObjectUI):
                 key = Interface(self.conn.get_backend())
                 key.type = Interface.INTERFACE_TYPE_ETHERNET
                 key.name = name
-            except Exception, e:
+            except Exception as e:
                 logging.debug("Error creating stub interface '%s': %s",
                     name, e)
                 continue
@@ -593,7 +592,7 @@ class vmmCreateInterface(vmmGObjectUI):
     #########################
 
     def get_config_interface_type(self):
-        return uiutil.get_list_selection(self.widget("interface-type"), 0)
+        return uiutil.get_list_selection(self.widget("interface-type"))
 
     def set_interface_name(self, name):
         if self.widget("interface-name-entry").get_visible():
@@ -610,7 +609,7 @@ class vmmCreateInterface(vmmGObjectUI):
             return self.widget("interface-name-label").get_text()
 
     def get_config_interface_startmode(self):
-        return uiutil.get_list_selection(self.widget("interface-startmode"), 0)
+        return uiutil.get_list_selection(self.widget("interface-startmode"))
 
     def get_config_selected_interfaces(self):
         iface_list = self.widget("interface-list")
@@ -675,7 +674,7 @@ class vmmCreateInterface(vmmGObjectUI):
         self.set_interface_name(name)
 
     def bond_monitor_mode_changed(self, src):
-        value = uiutil.get_list_selection(src, 1)
+        value = uiutil.get_list_selection(src, column=1)
         bond_pages = self.widget("bond-pages")
 
         if value == "arpmon":
@@ -709,8 +708,9 @@ class vmmCreateInterface(vmmGObjectUI):
         self.widget("bridge-config-label").set_text(txt)
 
     def update_bond_desc(self):
-        mode = uiutil.get_list_selection(self.widget("bond-mode"), 0)
-        mon = uiutil.get_list_selection(self.widget("bond-monitor-mode"), 1)
+        mode = uiutil.get_list_selection(self.widget("bond-mode"))
+        mon = uiutil.get_list_selection(
+            self.widget("bond-monitor-mode"), column=1)
 
         txt = mode
         if mon:
@@ -724,7 +724,7 @@ class vmmCreateInterface(vmmGObjectUI):
 
         if is_manual:
             if ipv4:
-                label += "IPv4: %s" % (ipv4.dhcp and "DHCP" or "Static")
+                label += "IPv4: %s" % (ipv4.dhcp and "DHCP" or _("Static"))
 
             if ipv6:
                 if label:
@@ -733,22 +733,22 @@ class vmmCreateInterface(vmmGObjectUI):
 
                 mode_label = ""
                 if ipv6.autoconf and ipv6.dhcp:
-                    mode_label += "Autoconf "
+                    mode_label += _("Autoconf") + " "
 
                 if ipv6.dhcp:
                     mode_label += "DHCP"
 
                 if not mode_label:
-                    mode_label = "Static"
+                    mode_label = _("Static")
 
                 label += mode_label
 
         else:
             if name:
-                label = "Copy configuration from '%s'" % name
+                label = _("Copy configuration from '%s'") % name
 
         if not label:
-            label = "No configuration"
+            label = _("No configuration")
 
         self.widget("ip-config-label").set_text(label)
 
@@ -764,7 +764,7 @@ class vmmCreateInterface(vmmGObjectUI):
     def build_ip_info(self):
         def build_ip(addr_str):
             if not addr_str:
-                return None, None
+                raise ValueError(_("Please enter an IP address"))
             ret = addr_str.rsplit("/", 1)
             address = ret[0]
             prefix = None
@@ -774,8 +774,8 @@ class vmmCreateInterface(vmmGObjectUI):
 
         is_manual = self.widget("ip-do-manual").get_active()
 
-        copy_row = uiutil.get_list_selection(
-            self.widget("ip-copy-interface-combo"), None)
+        copy_row = uiutil.get_list_selected_row(
+            self.widget("ip-copy-interface-combo"))
 
         v4_mode = self.widget("ipv4-mode").get_active()
         v4_addr = self.widget("ipv4-address").get_text()
@@ -792,9 +792,14 @@ class vmmCreateInterface(vmmGObjectUI):
         ipv6 = None
 
         if not is_manual:
-            if copy_row[1] and copy_row[2]:
-                copy_name = copy_row[1].get_name()
-                proto_xml = copy_row[1].get_protocol_xml()
+            copy_vmmiface = copy_row[1]
+            copy_cancopy = copy_row[2]
+            if copy_vmmiface and copy_cancopy:
+                copy_name = copy_vmmiface.get_name()
+                # We always want the inactive protocol XML, which
+                # will list the on disk config, not the run time config,
+                # which doesn't list DHCP
+                proto_xml = copy_vmmiface.get_protocol_xml(inactive=True)
 
         else:
             # Build IPv4 Info
@@ -818,9 +823,10 @@ class vmmCreateInterface(vmmGObjectUI):
                 if not ipv6.dhcp:
                     if v6_gate:
                         ipv6.gateway = v6_gate
-                    addr, prefix = build_ip(v4_addr)
-                    if addr:
-                        ipv6.add_ip(addr, prefix)
+                    for v6_addr in v6_addrlist:
+                        addr, prefix = build_ip(v6_addr)
+                        if addr:
+                            ipv6.add_ip(addr, prefix)
 
         return [is_manual, copy_name, ipv4, ipv6, proto_xml]
 
@@ -843,6 +849,7 @@ class vmmCreateInterface(vmmGObjectUI):
         row[0] = new_text
 
     def ipv6_address_selected(self, src=None):
+        ignore = src
         treepath = self.get_config_ipv6_address_selection()
         has_selection = (treepath is not None)
 
@@ -854,6 +861,7 @@ class vmmCreateInterface(vmmGObjectUI):
     #######################
 
     def back(self, src):
+        ignore = src
         notebook = self.widget("pages")
         curpage = notebook.get_current_page()
         notebook.set_current_page(curpage - 1)
@@ -899,7 +907,7 @@ class vmmCreateInterface(vmmGObjectUI):
             elif pagenum == PAGE_DETAILS:
                 return self.validate_details_page()
 
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Uncaught error validating install "
                                 "parameters: %s") % str(e))
             return
@@ -928,8 +936,19 @@ class vmmCreateInterface(vmmGObjectUI):
             if (itype == Interface.INTERFACE_TYPE_BRIDGE or
                 itype == Interface.INTERFACE_TYPE_BOND):
                 for row in ifaces:
-                    child = Interface(self.conn.get_backend(),
-                        parsexml=row[INTERFACE_ROW_KEY].get_xml_config())
+                    if row[INTERFACE_ROW_IS_DEFINED]:
+                        vmmiface = self.conn.get_interface(
+                            row[INTERFACE_ROW_NAME])
+
+                        # Use the inactive XML, which drops a bunch
+                        # elements that might cause netcf to choke on
+                        # for a sub-interface
+                        xml = vmmiface.get_xmlobj(
+                            inactive=True).get_xml_config()
+                    else:
+                        xml = row[INTERFACE_ROW_KEY].get_xml_config()
+
+                    child = Interface(self.conn.get_backend(), parsexml=xml)
                     iobj.add_interface(child)
                 check_conflict = True
 
@@ -956,13 +975,14 @@ class vmmCreateInterface(vmmGObjectUI):
                           "configured:\n\n%s\n\nUsing these may overwrite "
                           "their existing configuration. Are you sure you "
                           "want to use the selected interface(s)?") %
-                          defined_ifaces)
+                        defined_ifaces)
                 if not ret:
                     return ret
 
             # Validate IP info (get_config validates for us)
             (is_manual, copy_name, ipv4,
              ipv6, proto_xml) = self.get_config_ip_info()
+            ignore = copy_name
 
             if is_manual:
                 if ipv4:
@@ -976,13 +996,13 @@ class vmmCreateInterface(vmmGObjectUI):
                         parsexml=proto.get_xml_config()))
 
             if itype == Interface.INTERFACE_TYPE_BRIDGE:
-                ret = self.validate_bridge(iobj, ifaces)
+                ret = self.validate_bridge(iobj)
             elif itype == Interface.INTERFACE_TYPE_BOND:
-                ret = self.validate_bond(iobj, ifaces)
+                ret = self.validate_bond(iobj)
             elif itype == Interface.INTERFACE_TYPE_VLAN:
-                ret = self.validate_vlan(iobj, ifaces)
+                ret = self.validate_vlan(iobj)
             elif itype == Interface.INTERFACE_TYPE_ETHERNET:
-                ret = self.validate_ethernet(iobj, ifaces)
+                ret = self.validate_ethernet(iobj)
 
             if not ret:
                 return ret
@@ -991,13 +1011,13 @@ class vmmCreateInterface(vmmGObjectUI):
             iobj.validate()
 
             self.interface = iobj
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(
                             _("Error setting interface parameters."), e)
 
         return True
 
-    def validate_bridge(self, iobj, ifaces):
+    def validate_bridge(self, iobj):
         delay = self.widget("bridge-delay").get_value()
         stp = self.widget("bridge-stp").get_active()
 
@@ -1007,11 +1027,12 @@ class vmmCreateInterface(vmmGObjectUI):
         return True
 
 
-    def validate_bond(self, iobj, ifaces):
-        mode = uiutil.get_list_selection(self.widget("bond-mode"), 1)
-        mon = uiutil.get_list_selection(self.widget("bond-monitor-mode"), 1)
-        arp_val = uiutil.get_list_selection(self.widget("arp-validate"), 0)
-        mii_car = uiutil.get_list_selection(self.widget("mii-carrier"), 0)
+    def validate_bond(self, iobj):
+        mode = uiutil.get_list_selection(self.widget("bond-mode"), column=1)
+        mon = uiutil.get_list_selection(
+            self.widget("bond-monitor-mode"), column=1)
+        arp_val = uiutil.get_list_selection(self.widget("arp-validate"))
+        mii_car = uiutil.get_list_selection(self.widget("mii-carrier"))
 
         # ARP params
         arp_int = self.widget("arp-interval").get_value()
@@ -1042,21 +1063,22 @@ class vmmCreateInterface(vmmGObjectUI):
         return True
 
 
-    def validate_vlan(self, iobj, ifaces):
+    def validate_vlan(self, iobj):
         idx = uiutil.spin_get_helper(self.widget("vlan-tag"))
 
         iobj.tag = int(idx)
         return True
 
 
-    def validate_ethernet(self, iobj, ifaces):
+    def validate_ethernet(self, iobj):
+        ignore = iobj
         return True
 
 
     def validate_ip_info(self):
         try:
             self.build_ip_info()
-        except Exception, e:
+        except Exception as e:
             self.err.show_err(_("Error validating IP configuration: %s") %
                               str(e))
             return False
@@ -1089,9 +1111,7 @@ class vmmCreateInterface(vmmGObjectUI):
     #####################
 
     def _finish_cb(self, error, details):
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         if error:
             error = _("Error creating interface: '%s'") % error
@@ -1102,6 +1122,8 @@ class vmmCreateInterface(vmmGObjectUI):
             self.close()
 
     def finish(self, src):
+        ignore = src
+
         # Validate the final page
         page = self.widget("pages").get_current_page()
         if self.validate(page) is not True:
@@ -1110,10 +1132,7 @@ class vmmCreateInterface(vmmGObjectUI):
         activate = self.widget("interface-activate").get_active()
 
         # Start the install
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.WATCH))
-
+        self.set_finish_cursor()
         progWin = vmmAsyncJob(self.do_install, [activate],
                               self._finish_cb, [],
                               _("Creating virtual interface"),
